@@ -1,47 +1,33 @@
 /*
-  This file is part of the kholidays library.
+    This file is part of the kholidays library.
 
-  Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
-  Copyright (c) 2004 Allen Winter <winter@kde.org>
-  Copyright (c) 2008 David Jarvie <djarvie@kde.org>
-  Copyright 2010 John Layt <john@layt.net>
+    SPDX-FileCopyrightText: 2001 Cornelius Schumacher <schumacher@kde.org>
+    SPDX-FileCopyrightText: 2004-2020 Allen Winter <winter@kde.org>
+    SPDX-FileCopyrightText: 2008 David Jarvie <djarvie@kde.org>
+    SPDX-FileCopyrightText: 2010 John Layt <john@layt.net>
 
-  The functions languageToCode(), codeToLanguage(), countryToCode() and codeToCounty() are
-  Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+    The functions languageToCode(), codeToLanguage(), countryToCode() and codeToCounty() are copyright by
+    SPDX-FileCopyrightText: 2014 Digia Plc and/or its subsidiary(-ies).
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
-  License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU Library General Public License for more details.
-
-  You should have received a copy of the GNU Library General Public License
-  along with this library; see the file COPYING.LIB.  If not, write to the
-  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-  Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "holidayregion.h"
 
-#include <QLocale>
+#include <QCoreApplication>
 #include <QDirIterator>
 #include <QFile>
 #include <QFileInfo>
+#include <QLocale>
 #include <QSharedData>
 #include <QStandardPaths>
-#include <QCoreApplication>
 
 #include "holiday_p.h"
 #include "parsers/plan2/holidayparserdriverplan_p.h"
 
 using namespace KHolidays;
 
-// Copied from qlocale_data_p.h and qlocale.h
-// Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+// Copied from qlocale_data_p.h and qlocale.h, see copyright information at top of file
 // To later be replaced by OpenCodes or public Qt api
 static const unsigned char country_code_list[] =
     "ZZ\0" // AnyCountry
@@ -736,12 +722,12 @@ static void initResources()
 static QStringList allHolidayFiles(const QString &location = QString())
 {
     initResources();
-    QStringList dirs{ QStringLiteral(":/org.kde.kholidays/plan2") };
-    dirs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation,
-                             QStringLiteral("kf5/libkholidays/plan2"),
-                             QStandardPaths::LocateDirectory);
+    QStringList dirs{QStringLiteral(":/org.kde.kholidays/plan2")};
+    dirs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, //
+                                      QStringLiteral("kf5/libkholidays/plan2"),
+                                      QStandardPaths::LocateDirectory);
     QStringList files;
-    for (const QString &dir : qAsConst(dirs)) {
+    for (const QString &dir : std::as_const(dirs)) {
         QDirIterator it(dir, QStringList() << QStringLiteral("holiday_") + location + '*');
         while (it.hasNext()) {
             files.push_back(it.next());
@@ -750,15 +736,18 @@ static QStringList allHolidayFiles(const QString &location = QString())
     return files;
 }
 
-class Q_DECL_HIDDEN HolidayRegion::HolidayRegionPrivate
+namespace KHolidays
+{
+class HolidayRegionPrivate : public QSharedData
 {
 public:
-    explicit HolidayRegionPrivate(const QString &regionCode) : mDriver(nullptr),
-        mRegionCode(regionCode)
+    explicit HolidayRegionPrivate(const QString &regionCode)
+        : mDriver(nullptr)
+        , mRegionCode(regionCode)
     {
         if (!mRegionCode.isEmpty()) {
-            auto file = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
-                                       QLatin1String("kf5/libkholidays/plan2/holiday_") + mRegionCode);
+            auto file = QStandardPaths::locate(QStandardPaths::GenericDataLocation, //
+                                               QLatin1String("kf5/libkholidays/plan2/holiday_") + mRegionCode);
             if (!file.isEmpty()) {
                 mHolidayFile.setFile(file);
             } else {
@@ -773,8 +762,9 @@ public:
         init();
     }
 
-    explicit HolidayRegionPrivate(const QFileInfo &regionFile) : mDriver(nullptr),
-        mHolidayFile(regionFile)
+    explicit HolidayRegionPrivate(const QFileInfo &regionFile)
+        : mDriver(nullptr)
+        , mHolidayFile(regionFile)
     {
         init();
     }
@@ -789,7 +779,6 @@ public:
         if (mHolidayFile.exists()) {
             mDriver = new HolidayParserDriverPlan(mHolidayFile.absoluteFilePath());
             if (mDriver) {
-
                 if (mRegionCode.isEmpty()) {
                     if (mHolidayFile.fileName().startsWith(QLatin1String("holiday_"))) {
                         mRegionCode = mHolidayFile.fileName().mid(8);
@@ -806,10 +795,11 @@ public:
         }
     }
 
-    HolidayParserDriver *mDriver;  // The parser driver for the holiday file
-    QString mRegionCode;           // region code of holiday region
-    QFileInfo mHolidayFile;        // file containing holiday data, or null
+    HolidayParserDriver *mDriver; // The parser driver for the holiday file
+    QString mRegionCode; // region code of holiday region
+    QFileInfo mHolidayFile; // file containing holiday data, or null
 };
+}
 
 HolidayRegion::HolidayRegion(const QString &regionCode)
     : d(new HolidayRegionPrivate(regionCode))
@@ -821,10 +811,12 @@ HolidayRegion::HolidayRegion(const QFileInfo &regionFile)
 {
 }
 
-HolidayRegion::~HolidayRegion()
-{
-    delete d;
-}
+HolidayRegion::HolidayRegion(const HolidayRegion &) = default;
+HolidayRegion::HolidayRegion(HolidayRegion &&) = default;
+HolidayRegion::~HolidayRegion() = default;
+
+HolidayRegion &HolidayRegion::operator=(const HolidayRegion &) = default;
+HolidayRegion &HolidayRegion::operator=(HolidayRegion &&) = default;
 
 QStringList HolidayRegion::regionCodes()
 {
@@ -882,7 +874,8 @@ QString HolidayRegion::name() const
     if (tempName.isEmpty()) {
         QStringList countryParts = countryCode().toLower().split(QLatin1Char('-'));
         const QString &country = countryParts.at(0);
-        QString regionName, typeName;
+        QString regionName;
+        QString typeName;
 
         if (country != QLatin1String("xx")) {
             if (countryParts.count() == 2) {
@@ -891,8 +884,36 @@ QString HolidayRegion::name() const
                 const QString &subdivision = countryParts.at(1);
                 if (country == QLatin1String("ca") && subdivision == QLatin1String("qc")) {
                     regionName = QCoreApplication::translate("HolidayRegion", "Quebec", "Canadian region");
-                } else if (country == QLatin1String("de") && subdivision == QLatin1String("by")) {
-                    regionName = QCoreApplication::translate("HolidayRegion", "Bavaria", "German region");
+                } else if (country == QLatin1String("de")) {
+                    if (subdivision == QLatin1String("by")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Bavaria", "German region");
+                    } else if (subdivision == QLatin1String("bb")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Brandenburg", "German region");
+                    } else if (subdivision == QLatin1String("be")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Berlin", "German region");
+                    } else if (subdivision == QLatin1String("bw")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Baden-Wuerttemberg", "German region");
+                    } else if (subdivision == QLatin1String("he")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Hesse", "German region");
+                    } else if (subdivision == QLatin1String("mv")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Mecklenburg-Hither Pomerania", "German region");
+                    } else if (subdivision == QLatin1String("ni")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Lower Saxony", "German region");
+                    } else if (subdivision == QLatin1String("nw")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, North Rhine-Westphalia", "German region");
+                    } else if (subdivision == QLatin1String("rp")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Rhineland-Palatinate", "German region");
+                    } else if (subdivision == QLatin1String("sh")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Schleswig-Holstein", "German region");
+                    } else if (subdivision == QLatin1String("sl")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Saarland", "German region");
+                    } else if (subdivision == QLatin1String("sn")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Saxony", "German region");
+                    } else if (subdivision == QLatin1String("st")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Saxony-Anhalt", "German region");
+                    } else if (subdivision == QLatin1String("th")) {
+                        regionName = QCoreApplication::translate("HolidayRegion", "Germany, Thuringia", "German region");
+                    }
                 } else if (country == QLatin1String("es") && subdivision == QLatin1String("ct")) {
                     regionName = QCoreApplication::translate("HolidayRegion", "Catalonia", "Spanish region");
                 } else if (country == QLatin1String("gb") && subdivision == QLatin1String("eaw")) {
@@ -916,23 +937,18 @@ QString HolidayRegion::name() const
                 } else if (country == QLatin1String("au") && subdivision == QLatin1String("sa")) {
                     regionName = QCoreApplication::translate("HolidayRegion", "South Australia", "Australian Region");
                 } else if (country == QLatin1String("au") && subdivision == QLatin1String("nt")) {
-                    regionName = QCoreApplication::translate("HolidayRegion",
-                                                             "Northern Territory", "Australian Region");
+                    regionName = QCoreApplication::translate("HolidayRegion", "Northern Territory", "Australian Region");
                 } else if (country == QLatin1String("au") && subdivision == QLatin1String("act")) {
-                    regionName = QCoreApplication::translate("HolidayRegion",
-                                                             "Australian Capital Territory", "Australian Region");
+                    regionName = QCoreApplication::translate("HolidayRegion", "Australian Capital Territory", "Australian Region");
                 } else if (country == QLatin1String("au") && subdivision == QLatin1String("wa")) {
-                    regionName = QCoreApplication::translate("HolidayRegion",
-                                                             "Western Australia", "Australian Region");
+                    regionName = QCoreApplication::translate("HolidayRegion", "Western Australia", "Australian Region");
                 } else if (country == QLatin1String("au") && subdivision == QLatin1String("tas")) {
-                    regionName = QCoreApplication::translate("HolidayRegion",
-                                                             "Tasmania", "Australian Region");
+                    regionName = QCoreApplication::translate("HolidayRegion", "Tasmania", "Australian Region");
                 } else if (country == QLatin1String("ba") && subdivision == QLatin1String("srp")) {
-                    regionName = QCoreApplication::translate("HolidayRegion",
-                                                             "Republic of Srpska", "Bosnian and Herzegovinian Region");
+                    regionName = QCoreApplication::translate("HolidayRegion", "Republic of Srpska", "Bosnian and Herzegovinian Region");
                 } else {
                     // TODO Note this does not give the current QLocale translation!
-                    regionName = QLocale::countryToString(codeToCountry(country));
+                    regionName = QLocale::countryToString(codeToCountry(country)) + QLatin1String(" (") + subdivision + QLatin1Char(')');
                 }
             } else {
                 // TODO Note this does not give the current QLocale translation!
@@ -940,7 +956,7 @@ QString HolidayRegion::name() const
             }
         }
 
-        //Cheat on type for now,take direct from region code until API is introduced in SC 4.6
+        // Cheat on type for now,take direct from region code until API is introduced in SC 4.6
         QStringList regionParts = regionCode().toLower().split(QLatin1Char('_'));
         if (regionParts.count() == 3) {
             const QString &type = regionParts.at(2);
@@ -1001,11 +1017,8 @@ QString HolidayRegion::name() const
 
         if (!regionName.isEmpty()) {
             if (!typeName.isEmpty()) {
-                tempName =
-                    QCoreApplication::translate(
-                        "HolidayRegion",
-                        "%1 - %2",
-                        "Holiday file display name, %1 = region name, %2 = holiday type").arg(regionName, typeName);
+                tempName = QCoreApplication::translate("HolidayRegion", "%1 - %2", "Holiday file display name, %1 = region name, %2 = holiday type")
+                               .arg(regionName, typeName);
             } else {
                 tempName = regionName;
             }
@@ -1054,7 +1067,32 @@ bool HolidayRegion::isValid(const QString &regionCode)
     return temp.isValid();
 }
 
+Holiday::List HolidayRegion::rawHolidays(const QDate &startDate, const QDate &endDate, const QString &category) const
+{
+    if (isValid()) {
+        return d->mDriver->parseHolidays(startDate, endDate, category);
+    } else {
+        return Holiday::List();
+    }
+}
+
+#if KHOLIDAYS_BUILD_DEPRECATED_SINCE(5, 95)
 Holiday::List HolidayRegion::holidays(const QDate &startDate, const QDate &endDate) const
+{
+    return rawHolidaysWithAstroSeasons(startDate, endDate);
+}
+#endif
+
+Holiday::List HolidayRegion::rawHolidays(const QDate &startDate, const QDate &endDate) const
+{
+    if (isValid()) {
+        return d->mDriver->parseRawHolidays(startDate, endDate);
+    } else {
+        return Holiday::List();
+    }
+}
+
+Holiday::List HolidayRegion::rawHolidaysWithAstroSeasons(const QDate &startDate, const QDate &endDate) const
 {
     if (isValid()) {
         return d->mDriver->parseHolidays(startDate, endDate);
@@ -1063,7 +1101,14 @@ Holiday::List HolidayRegion::holidays(const QDate &startDate, const QDate &endDa
     }
 }
 
+#if KHOLIDAYS_BUILD_DEPRECATED_SINCE(5, 95)
 Holiday::List HolidayRegion::holidays(const QDate &date) const
+{
+    return rawHolidaysWithAstroSeasons(date);
+}
+#endif
+
+Holiday::List HolidayRegion::rawHolidaysWithAstroSeasons(const QDate &date) const
 {
     if (isValid()) {
         return d->mDriver->parseHolidays(date);
@@ -1072,7 +1117,14 @@ Holiday::List HolidayRegion::holidays(const QDate &date) const
     }
 }
 
+#if KHOLIDAYS_BUILD_DEPRECATED_SINCE(5, 95)
 Holiday::List HolidayRegion::holidays(int calendarYear) const
+{
+    return rawHolidaysWithAstroSeasons(calendarYear);
+}
+#endif
+
+Holiday::List HolidayRegion::rawHolidaysWithAstroSeasons(int calendarYear) const
 {
     if (isValid()) {
         return d->mDriver->parseHolidays(calendarYear);
@@ -1083,7 +1135,7 @@ Holiday::List HolidayRegion::holidays(int calendarYear) const
 
 bool HolidayRegion::isHoliday(const QDate &date) const
 {
-    const Holiday::List holidayList = holidays(date);
+    const Holiday::List holidayList = rawHolidaysWithAstroSeasons(date);
     if (!holidayList.isEmpty()) {
         for (const KHolidays::Holiday &holiday : holidayList) {
             if (holiday.dayType() == Holiday::NonWorkday) {
@@ -1094,6 +1146,17 @@ bool HolidayRegion::isHoliday(const QDate &date) const
     return false;
 }
 
+static bool maybeCountry(QStringView holidayId, QStringView country)
+{
+    if (country.isEmpty()) {
+        return false;
+    }
+    if (holidayId.size() < 2 || country.size() < 2 || (country.size() > 2 && country[2] != QLatin1Char('-'))) {
+        return true; // not the format we expect, check the content
+    }
+    return holidayId.startsWith(country.left(2));
+}
+
 QString HolidayRegion::defaultRegionCode(const QString &country, const QString &language)
 {
     // Try to match against the users country and language, or failing that the language country.
@@ -1105,12 +1168,17 @@ QString HolidayRegion::defaultRegionCode(const QString &country, const QString &
     // Locale language can have a country code embedded in it e.g. en_GB, which we can try use if
     // no country set, but a lot of countries use en_GB so it's a lower priority option
 
-    QString localeCountry, localeLanguage, localeLanguageCountry;
+    QString localeCountry;
+    QString localeSubdivision;
+    QString localeLanguage;
+    QString localeLanguageCountry;
 
     if (country.isEmpty()) {
         localeCountry = countryToCode(QLocale().country()).toLower();
     } else {
         localeCountry = country.toLower();
+        const auto idx = localeCountry.indexOf(QLatin1Char('-'));
+        localeSubdivision = idx > 0 ? localeCountry.left(idx) : localeCountry;
     }
 
     if (language.isEmpty()) {
@@ -1123,58 +1191,65 @@ QString HolidayRegion::defaultRegionCode(const QString &country, const QString &
         localeLanguageCountry = localeLanguage.split(QLatin1Char('_')).at(1);
     }
 
-    QString countryAndLanguageMatch, countryOnlyMatch, subdivisionAndLanguageMatch,
-            subdivisionOnlyMatch, languageCountryAndLanguageMatch, languageCountryOnlyMatch,
-            languageSubdivisionAndLanguageMatch, languageSubdivisionOnlyMatch;
+    QString countryAndLanguageMatch;
+    QString countryOnlyMatch;
+    QString subdivisionAndLanguageMatch;
+    QString subdivisionOnlyMatch;
+    QString languageCountryAndLanguageMatch;
+    QString languageCountryOnlyMatch;
+    QString languageSubdivisionAndLanguageMatch;
+    QString languageSubdivisionOnlyMatch;
 
     const QStringList regionList = KHolidays::HolidayRegion::regionCodes();
-    for (const QString &regionCode : regionList) {
-        QString regionCountry = KHolidays::HolidayRegion::countryCode(regionCode).toLower();
+    for (const QString &aRegionCode : regionList) {
+        // avoid expensive parsing in most cases by leveraging the country being in the file name
+        if (!maybeCountry(aRegionCode, localeCountry) && !maybeCountry(aRegionCode, localeLanguageCountry)) {
+            continue;
+        }
+
+        const auto hr = KHolidays::HolidayRegion(aRegionCode);
+        QString regionCountry = hr.countryCode().toLower();
         QString regionSubdivisionCountry;
         if (regionCountry.split(QLatin1Char('-')).count() > 1) {
             regionSubdivisionCountry = regionCountry.split(QLatin1Char('-')).at(0);
+        } else {
+            regionSubdivisionCountry = regionCountry;
         }
-        QString regionLanguage = KHolidays::HolidayRegion::languageCode(regionCode).toLower();
+        QString regionLanguage = hr.languageCode().toLower();
 
         if (regionCountry == localeCountry && regionLanguage == localeLanguage) {
-            countryAndLanguageMatch = regionCode;
-            break; // exact match so don't look further
+            // exact match so don't look further
+            return aRegionCode;
+        } else if (regionCountry == localeSubdivision && regionLanguage == localeLanguage) {
+            countryAndLanguageMatch = aRegionCode;
         } else if (regionCountry == localeCountry) {
             if (countryOnlyMatch.isEmpty()) {
-                countryOnlyMatch = regionCode;
+                countryOnlyMatch = aRegionCode;
             }
-        } else if (!regionSubdivisionCountry.isEmpty() &&
-                   regionSubdivisionCountry == localeCountry &&
-                   regionLanguage == localeLanguage) {
+        } else if (!regionSubdivisionCountry.isEmpty() && regionSubdivisionCountry == localeSubdivision && regionLanguage == localeLanguage) {
             if (subdivisionAndLanguageMatch.isEmpty()) {
-                subdivisionAndLanguageMatch = regionCode;
+                subdivisionAndLanguageMatch = aRegionCode;
             }
-        } else if (!regionSubdivisionCountry.isEmpty() && regionSubdivisionCountry == localeCountry) {
+        } else if (!regionSubdivisionCountry.isEmpty() && regionSubdivisionCountry == localeSubdivision) {
             if (subdivisionOnlyMatch.isEmpty()) {
-                subdivisionOnlyMatch = regionCode;
+                subdivisionOnlyMatch = aRegionCode;
             }
-        } else if (!localeLanguageCountry.isEmpty() &&
-                   regionCountry == localeLanguageCountry &&
-                   regionLanguage == localeLanguage) {
+        } else if (!localeLanguageCountry.isEmpty() && regionCountry == localeLanguageCountry && regionLanguage == localeLanguage) {
             if (languageCountryAndLanguageMatch.isEmpty()) {
-                languageCountryAndLanguageMatch = regionCode;
+                languageCountryAndLanguageMatch = aRegionCode;
             }
         } else if (!localeLanguageCountry.isEmpty() && regionCountry == localeLanguageCountry) {
             if (languageCountryOnlyMatch.isEmpty()) {
-                languageCountryOnlyMatch = regionCode;
+                languageCountryOnlyMatch = aRegionCode;
             }
-        } else if (!regionSubdivisionCountry.isEmpty() &&
-                   !localeLanguageCountry.isEmpty() &&
-                   regionSubdivisionCountry == localeLanguageCountry &&
-                   regionLanguage == localeLanguage) {
+        } else if (!regionSubdivisionCountry.isEmpty() && !localeLanguageCountry.isEmpty() && regionSubdivisionCountry == localeLanguageCountry
+                   && regionLanguage == localeLanguage) {
             if (languageSubdivisionAndLanguageMatch.isEmpty()) {
-                languageSubdivisionAndLanguageMatch = regionCode;
+                languageSubdivisionAndLanguageMatch = aRegionCode;
             }
-        } else if (!regionSubdivisionCountry.isEmpty() &&
-                   !localeLanguageCountry.isEmpty() &&
-                   regionSubdivisionCountry == localeLanguageCountry) {
+        } else if (!regionSubdivisionCountry.isEmpty() && !localeLanguageCountry.isEmpty() && regionSubdivisionCountry == localeLanguageCountry) {
             if (languageSubdivisionOnlyMatch.isEmpty()) {
-                languageSubdivisionOnlyMatch = regionCode;
+                languageSubdivisionOnlyMatch = aRegionCode;
             }
         }
     }
