@@ -18,33 +18,31 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
+#include "holiday_p.h"
 #include "holidayparserdriverplan_p.h"
-#include "holidayscannerplan_p.h"
 #include "holidayparserplan.hpp"
-
-#include <sstream>
+#include "holidayscannerplan_p.h"
+#include <kholidays_debug.h>
 
 #include <QFileInfo>
 
-#include <qdebug.h>
+#include <sstream>
 
-#include "holiday_p.h"
-
-#define LAST        99999
-#define ANY       (-99999)
-#define BEFORE        (-1)
-#define AFTER           1
+#define LAST 99999
+#define ANY (-99999)
+#define BEFORE (-1)
+#define AFTER 1
 
 using namespace KHolidays;
 
 HolidayParserDriverPlan::HolidayParserDriverPlan(const QString &planFilePath)
-    : HolidayParserDriver(planFilePath),
-      m_traceParsing(false),
-      m_traceScanning(false),
-      m_parseMetadataOnly(false),
-      m_eventYear(0),
-      m_eventMonth(0),
-      m_eventDay(0)
+    : HolidayParserDriver(planFilePath)
+    , m_traceParsing(false)
+    , m_traceScanning(false)
+    , m_parseMetadataOnly(false)
+    , m_eventYear(0)
+    , m_eventMonth(0)
+    , m_eventDay(0)
 {
     QFile holidayFile(filePath());
     if (holidayFile.open(QIODevice::ReadOnly)) {
@@ -66,25 +64,24 @@ HolidayParserDriverPlan::~HolidayParserDriverPlan()
     delete m_fileToParse;
 }
 
-//TODO Figure why it doesn't compile
+// TODO Figure why it doesn't compile
 void HolidayParserDriverPlan::error(const KHolidays::location &errorLocation, const QString &errorMessage)
 {
     Q_UNUSED(errorLocation);
-    //std::cerr << errorLocation << " : " << errorMessage;  //Doesn't work???
-    //qDebug() << errorLocation << " : " << errorMessage;  //Doesn't work???
-    qDebug() << errorMessage;
+    // std::cerr << errorLocation << " : " << errorMessage;  //Doesn't work???
+    // qDebug() << errorLocation << " : " << errorMessage;  //Doesn't work???
+    qCDebug(KHOLIDAYS_LOG) << errorMessage;
 }
 
 void HolidayParserDriverPlan::error(const QString &errorMessage)
 {
-    qDebug() << errorMessage;
+    qCDebug(KHOLIDAYS_LOG) << errorMessage;
 }
 
 void HolidayParserDriverPlan::parse()
 {
     // Parse the file using every calendar system in the file
-    for (const QString &calendarType : qAsConst(m_fileCalendarTypes)) {
-
+    for (const QString &calendarType : std::as_const(m_fileCalendarTypes)) {
         // Cater for events defined in other Calendar Systems where request year could cover 2 or 3 event years
         // Perhaps also parse year before and year after to allow events to span years or shift to other year?
         setParseCalendar(calendarType);
@@ -92,7 +89,6 @@ void HolidayParserDriverPlan::parse()
 
         // Generate all events for this calendar in the required year(s)
         for (m_parseYear = m_parseStartYear; m_parseYear <= m_parseEndYear; ++m_parseYear) {
-
             m_parseYearStart = m_parseCalendar.firstDayOfYear(m_parseYear);
             m_parseYearEaster = easter(m_parseYear);
             m_parseYearPascha = pascha(m_parseYear);
@@ -102,7 +98,6 @@ void HolidayParserDriverPlan::parse()
 
             m_parser->parse();
         }
-
     }
 }
 
@@ -175,18 +170,17 @@ std::string *HolidayParserDriverPlan::fileToParse() const
 // Adjust month numbering for Hebrew civil calendar leap month
 int HolidayParserDriverPlan::adjustedMonthNumber(int month)
 {
-    if (m_eventCalendarType != QLatin1String("hebrew") ||               // Only adjust Hebrew months
-            m_parseCalendarType != QLatin1String("hebrew") ||
-            !m_parseCalendar.isLeapYear(m_parseYear) ||    // Only adjust in leap year
-            month < 6) {                                    // Only adjust from Adar onwards
+    if (m_eventCalendarType != QLatin1String("hebrew") || // Only adjust Hebrew months
+        m_parseCalendarType != QLatin1String("hebrew") || !m_parseCalendar.isLeapYear(m_parseYear) || // Only adjust in leap year
+        month < 6) { // Only adjust from Adar onwards
         return month;
     }
 
-    if (month == 13) {   // Adar I
+    if (month == 13) { // Adar I
         return 6;
     }
 
-    if (month == 14) {   // Adar II
+    if (month == 14) { // Adar II
         return 7;
     }
 
@@ -228,9 +222,12 @@ QDate HolidayParserDriverPlan::easter(int year)
     if (m_parseCalendar.calendarSystem() != QCalendarSystem::GregorianCalendar) {
         return QDate();
     }
+    if (year < 0) {
+        return QDate();
+    }
 
     // Algorithm taken from Tondering
-    // http://www.tondering.dk/claus/cal/node3.html#SECTION003137000000000000000
+    // https://www.tondering.dk/claus/cal/easter.php
     int g = year % 19;
     int c = year / 100;
     int h = (c - (c / 4) - (((8 * c) + 13) / 25) + (19 * g) + 15) % 30;
@@ -245,10 +242,13 @@ QDate HolidayParserDriverPlan::easter(int year)
 
 QDate HolidayParserDriverPlan::pascha(int year)
 {
-    if (m_parseCalendar.calendarSystem() == QCalendarSystem::GregorianCalendar ||
-        m_parseCalendar.calendarSystem() == QCalendarSystem::JulianCalendar) {
+    if (year < 0) {
+        return QDate();
+    }
+
+    if (m_parseCalendar.calendarSystem() == QCalendarSystem::GregorianCalendar || m_parseCalendar.calendarSystem() == QCalendarSystem::JulianCalendar) {
         // Algorithm taken from Tondering
-        // http://www.tondering.dk/claus/cal/node3.html#SECTION003137000000000000000
+        // https://www.tondering.dk/claus/cal/easter.php#orthodoxeast
         // Gives Orthodox Easter in the Julian Calendar, need to convert afterwards to Gregorian if needed
         int g = year % 19;
         int i = ((19 * g) + 15) % 30;
@@ -325,7 +325,7 @@ QString HolidayParserDriverPlan::systemToType(QCalendarSystem::CalendarSystem ca
 // Return the jd of an existing event, assumes unique names and correct order in file
 int HolidayParserDriverPlan::julianDayFromEventName(const QString &eventName)
 {
-    for (const KHolidays::Holiday &thisHoliday : qAsConst(m_resultList)) {
+    for (const KHolidays::Holiday &thisHoliday : std::as_const(m_resultList)) {
         if (thisHoliday.name() == eventName) {
             return thisHoliday.observedStartDate().toJulianDay();
         }
@@ -364,7 +364,7 @@ int HolidayParserDriverPlan::julianDayFromMonthDay(int month, int day)
 // Return jd of weekday relative to a Julian Day number
 int HolidayParserDriverPlan::julianDayFromRelativeWeekday(int occurrence, int weekday, int jd)
 {
-    if (occurrence == ANY) {    /* Should never get this, convert to AFTER instead */
+    if (occurrence == ANY) { /* Should never get this, convert to AFTER instead */
         occurrence = AFTER;
     }
 
@@ -385,12 +385,12 @@ int HolidayParserDriverPlan::julianDayFromRelativeWeekday(int occurrence, int we
     return jd + weekday - thisWeekday + (occurrence * 7);
 }
 
-// Return jd of weekday occurence in a given month and day in the parse year
+// Return jd of weekday occurrence in a given month and day in the parse year
 int HolidayParserDriverPlan::julianDayFromWeekdayInMonth(int occurrence, int weekday, int month)
 {
-    if (occurrence == LAST) {    // Is weekday on or before last day of month
+    if (occurrence == LAST) { // Is weekday on or before last day of month
         return julianDayFromRelativeWeekday(BEFORE, weekday, julianDay(m_parseYear, month, m_parseCalendar.daysInMonth(m_parseYear, month)));
-    } else {  // Is nth weekday on or after first day of month
+    } else { // Is nth weekday on or after first day of month
         return julianDayFromRelativeWeekday(occurrence, weekday, julianDay(m_parseYear, month, 1));
     }
 }
@@ -405,39 +405,39 @@ void HolidayParserDriverPlan::setParseCalendar(const QString &calendarType)
     setParseCalendar(typeToSystem(calendarType));
 }
 
-void  HolidayParserDriverPlan::setFileCountryCode(const QString &countryCode)
+void HolidayParserDriverPlan::setFileCountryCode(const QString &countryCode)
 {
     m_fileCountryCode = countryCode;
 }
 
-void  HolidayParserDriverPlan::setFileLanguageCode(const QString &languageCode)
+void HolidayParserDriverPlan::setFileLanguageCode(const QString &languageCode)
 {
     m_fileLanguageCode = languageCode;
 }
 
-void  HolidayParserDriverPlan::setFileName(const QString &name)
+void HolidayParserDriverPlan::setFileName(const QString &name)
 {
     m_fileName = name;
 }
 
-void  HolidayParserDriverPlan::setFileDescription(const QString &description)
+void HolidayParserDriverPlan::setFileDescription(const QString &description)
 {
     m_fileDescription = description;
 }
 
-void  HolidayParserDriverPlan::setEventName(const QString &eventName)
+void HolidayParserDriverPlan::setEventName(const QString &eventName)
 {
     // Assume if setting an event name then is start of new event line, so clear categories
     m_eventCategories.clear();
     m_eventName = eventName;
 }
 
-void  HolidayParserDriverPlan::setEventCategory(const QString &category)
+void HolidayParserDriverPlan::setEventCategory(const QString &category)
 {
     m_eventCategories.append(category);
 }
 
-void  HolidayParserDriverPlan::setEventCalendarType(const QString &calendarType)
+void HolidayParserDriverPlan::setEventCalendarType(const QString &calendarType)
 {
     m_eventCalendarType = calendarType;
     if (m_parseMetadataOnly && !m_fileCalendarTypes.contains(calendarType)) {
@@ -445,14 +445,14 @@ void  HolidayParserDriverPlan::setEventCalendarType(const QString &calendarType)
     }
 }
 
-void  HolidayParserDriverPlan::setEventDate(int eventYear, int eventMonth, int eventDay)
+void HolidayParserDriverPlan::setEventDate(int eventYear, int eventMonth, int eventDay)
 {
     m_eventYear = eventYear;
     m_eventMonth = eventMonth;
     m_eventDay = eventDay;
 }
 
-void  HolidayParserDriverPlan::setEventDate(int jd)
+void HolidayParserDriverPlan::setEventDate(int jd)
 {
     julianDayToDate(jd, &m_eventYear, &m_eventMonth, &m_eventDay);
 }
@@ -488,13 +488,12 @@ void HolidayParserDriverPlan::setFromWeekdayInMonth(int occurrence, int weekday,
 
     // Generate all events in the required event month(s)
     for (int thisMonth = startMonth; thisMonth <= endMonth; ++thisMonth) {
-
         if (m_parseCalendar.isValid(m_parseYear, thisMonth, 1)) {
             int startOccurrence, endOccurrence;
-            if (occurrence == ANY) {    // Generate 1st through 5th weekdays, assumes no month with > 35 days
+            if (occurrence == ANY) { // Generate 1st through 5th weekdays, assumes no month with > 35 days
                 startOccurrence = 1;
                 endOccurrence = 5;
-            } else {  // Generate just nth or LAST weekday
+            } else { // Generate just nth or LAST weekday
                 startOccurrence = occurrence;
                 endOccurrence = occurrence;
             }
@@ -510,7 +509,6 @@ void HolidayParserDriverPlan::setFromWeekdayInMonth(int occurrence, int weekday,
                 }
             }
         }
-
     }
 }
 
@@ -528,20 +526,20 @@ void HolidayParserDriverPlan::setFromRelativeWeekday(int occurrence, int weekday
     }
 
     int thisYear;
-    if (m_eventYear == ANY) {    // Generate the parse year
+    if (m_eventYear == ANY) { // Generate the parse year
         thisYear = m_parseYear;
-    } else {  // Generate a specific event year
+    } else { // Generate a specific event year
         thisYear = m_eventYear;
     }
 
     int startMonth, endMonth;
-    if (m_eventMonth == LAST) {    // Generate just the last month
+    if (m_eventMonth == LAST) { // Generate just the last month
         startMonth = m_parseCalendar.monthsInYear(thisYear);
         endMonth = startMonth;
-    } else if (m_eventMonth == ANY) {    // Generate every month
+    } else if (m_eventMonth == ANY) { // Generate every month
         startMonth = 1;
         endMonth = m_parseCalendar.monthsInYear(thisYear);
-    } else {  // Generate just the specific event month
+    } else { // Generate just the specific event month
         startMonth = m_eventMonth;
         endMonth = m_eventMonth;
     }
@@ -549,15 +547,14 @@ void HolidayParserDriverPlan::setFromRelativeWeekday(int occurrence, int weekday
     // Generate all events in the required month(s)
     int thisMonth;
     for (thisMonth = startMonth; thisMonth <= endMonth; ++thisMonth) {
-
         int startDay, endDay;
-        if (m_eventDay == LAST) {    // Generate just the last day in the month
+        if (m_eventDay == LAST) { // Generate just the last day in the month
             startDay = m_parseCalendar.daysInMonth(thisYear, thisMonth);
             endDay = startDay;
-        } else if (m_eventDay == ANY) {    // Generate every day in the month
+        } else if (m_eventDay == ANY) { // Generate every day in the month
             startDay = 1;
             endDay = m_parseCalendar.daysInMonth(thisYear, thisMonth);
-        } else {  // Generate just the specific event day
+        } else { // Generate just the specific event day
             startDay = m_eventDay;
             endDay = m_eventDay;
         }
@@ -569,7 +566,6 @@ void HolidayParserDriverPlan::setFromRelativeWeekday(int occurrence, int weekday
                 setEvent(relativeJd + offset, 0, duration);
             }
         }
-
     }
 }
 
@@ -614,49 +610,46 @@ void HolidayParserDriverPlan::setFromDate(int offset, int condition, int duratio
     }
 
     int thisYear;
-    if (m_eventYear == ANY) {    // Generate the parse year
+    if (m_eventYear == ANY) { // Generate the parse year
         thisYear = m_parseYear;
-    } else {  // Generate a specific event year
+    } else { // Generate a specific event year
         thisYear = m_eventYear;
     }
 
     int startMonth, endMonth;
-    if (m_eventMonth == LAST) {    // Generate just the last month
+    if (m_eventMonth == LAST) { // Generate just the last month
         startMonth = m_parseCalendar.monthsInYear(thisYear);
         endMonth = startMonth;
-    } else if (m_eventMonth == ANY) {    // Generate every month
+    } else if (m_eventMonth == ANY) { // Generate every month
         startMonth = 1;
         endMonth = m_parseCalendar.monthsInYear(thisYear);
-    } else {  // Generate just the specific event month
+    } else { // Generate just the specific event month
         startMonth = m_eventMonth;
         endMonth = m_eventMonth;
     }
 
     // Generate all events in the required month(s)
     for (int thisMonth = startMonth; thisMonth <= endMonth; ++thisMonth) {
-
         int startDay, endDay;
-        if (m_eventDay == LAST) {    // Generate just the last day in the month
+        if (m_eventDay == LAST) { // Generate just the last day in the month
             startDay = m_parseCalendar.daysInMonth(thisYear, thisMonth);
             endDay = startDay;
-        } else if (m_eventDay == ANY) {    // Generate every day in the month
+        } else if (m_eventDay == ANY) { // Generate every day in the month
             startDay = 1;
             endDay = m_parseCalendar.daysInMonth(thisYear, thisMonth);
-        } else {  // Generate just the specific event day
+        } else { // Generate just the specific event day
             startDay = m_eventDay;
             endDay = m_eventDay;
         }
 
         // Generate all events on the required day(s)
         for (int thisDay = startDay; thisDay <= endDay; ++thisDay) {
-
             if (m_parseCalendar.isValid(thisYear, thisMonth, thisDay)) {
-                setEvent(julianDay(thisYear, thisMonth, thisDay) + offset,
-                         conditionalOffset(thisYear, thisMonth, thisDay, condition), duration);
+                setEvent(julianDay(thisYear, thisMonth, thisDay) + offset, //
+                         conditionalOffset(thisYear, thisMonth, thisDay, condition), //
+                         duration);
             }
-
         }
-
     }
 }
 
@@ -717,9 +710,9 @@ void HolidayParserDriverPlan::setEvent(int jd, int observeOffset, int duration)
 void HolidayParserDriverPlan::addHoliday(const QDate &observedDate, int duration)
 {
     // Only set if event falls in requested date range, i.e. either starts or ends during range
-    if (m_parseCalendar.isValid(observedDate) &&
-            observedDate <= m_requestEnd &&
-            observedDate.addDays(duration - 1) >= m_requestStart) {
+    if (m_parseCalendar.isValid(observedDate) //
+        && observedDate <= m_requestEnd //
+        && observedDate.addDays(duration - 1) >= m_requestStart) {
         KHolidays::Holiday holiday;
         holiday.d->mObservedDate = observedDate;
         holiday.d->mDuration = duration;
